<?php

namespace Drupal\gotem\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configuration form for the Gotem module.
 */
class GotemSettingsForm extends ConfigFormBase {

  /**
   * The entity field manager service.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * Constructs a new GotemSettingsForm.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager service.
   */
  public function __construct(EntityFieldManagerInterface $entity_field_manager) {
    $this->entityFieldManager = $entity_field_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_field.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['gotem.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'gotem_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('gotem.settings');
    $api_key = \Drupal::state()->get('gotem_content_moderation.api_key');

    // API key field, disabled if a value exists
    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('OpenAI API Key'),
      '#default_value' => $this->getPartialApiKey($api_key),
      '#required' => TRUE,
      '#description' => $this->t('Enter your OpenAI API key here.'),
      '#maxlength' => 256,
      '#size' => 64,
      '#disabled' => !empty($api_key),
    ];

    // Reset API key button
    if (!empty($api_key)) {
      $form['reset_api_key'] = [
        '#type' => 'submit',
        '#value' => $this->t('Reset API Key'),
        '#submit' => ['::resetApiKey'],
        '#button_type' => 'danger',
      ];
    }

    // Moderated fields selection
    $form['moderated_fields'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Fields to Moderate (Content Entities)'),
      '#description' => $this->t('Select the fields you want to moderate using the OpenAI Moderation API.'),
      '#options' => $this->getModeratableFields('node'),
      '#default_value' => $config->get('moderated_fields') ?: [],
      '#required' => TRUE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Reset the API key.
   */
  public function resetApiKey(array &$form, FormStateInterface $form_state) {
    \Drupal::state()->delete('gotem_content_moderation.api_key');
    \Drupal::messenger()->addMessage($this->t('The API key has been reset.'));
    $form_state->setRebuild(TRUE);
  }

  /**
   * Retrieves fields eligible for moderation for a given entity type.
   *
   * @param string $entity_type
   *   The entity type ('node' or 'media').
   *
   * @return array
   *   Field machine names and their labels.
   */
  protected function getModeratableFields($entity_type) {
    $fields = [];
    $bundle_info = \Drupal::service('entity_type.bundle.info')->getBundleInfo($entity_type);

    foreach ($bundle_info as $bundle_name => $bundle_data) {
      $field_definitions = $this->entityFieldManager->getFieldDefinitions($entity_type, $bundle_name);

      foreach ($field_definitions as $field_name => $field_definition) {
        $field_type = $field_definition->getType();
        if (in_array($field_type, ['text', 'text_long', 'string', 'string_long'])) {
          $label = $field_definition->getLabel();
          $fields[$field_name] = $label;
        }
      }
    }

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $api_key = $form_state->getValue('api_key');
    $selected_fields = array_filter($form_state->getValue('moderated_fields'));

    \Drupal::state()->set('gotem_content_moderation.api_key', $api_key);

    $this->config('gotem.settings')
      ->set('moderated_fields', array_keys($selected_fields))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Display a partial (masked) API key.
   *
   * @param string $api_key
   *   The full API key.
   *
   * @return string
   *   The masked API key.
   */
  protected function getPartialApiKey($api_key) {
    return $api_key ? substr($api_key, 0, 15) . str_repeat('*', strlen($api_key) - 4) : '';
  }

}
